/*
*      _                      _____               _                _  _              
*     | |                    / ____|             | |              | || |            
*   __| | _ __  ___   _ __  | |      ___   _ __  | |_  _ __  ___  | || |  ___  _ __ 
*  / _` || '__|/ _ \ | '_ \ | |     / _ \ | '_ \ | __|| '__|/ _ \ | || | / _ \| '__| 
* | (_| || |  | (_) || |_) || |____| (_) || | | || |_ | |  | (_) || || ||  __/| |   
*  \__,_||_|   \___/ | .__/  \_____|\___/ |_| |_| \__||_|   \___/ |_||_| \___||_|      
*                    | |                                                                                                      
*                    |_|                                                                            
*
* By Martyn Currey.  www.dropController.com  2022-07-11
* Copyright Martyun Currey 2022. 
* 
* This is free software. No restrictions on updating and changing it.
* Can be freely distributed as long as payment is not requested.
* 
* 
* dropControllerV3_drops_001
* created for Evan Politi.
* 
* Works with the Windows app or via serial commands
* Not compatible with the Android app at this time.
* 
* Designed to create a stream of drops from a single valve.
* number of drops limit is 5 digit, 0 - 99999 drops
* drop size limit is 5 digit, 0 - 99999 drops
* delay between drops limit is 5 digit, 0 - 99999 drops
* 
* 
* debug information is sent through the software serial to the connected wireless device.
* Needs to be used with a Bluetooth Classic app such as Bluetooth Terminal. This is third party software and not 
* supported or otherwise endoresed by dropController or Martyn Currey.
* 
*
*
* 
* 
*   _____   _             
*  |  __ \ (_)            
*  | |__) | _  _ __   ___ 
*  |  ___/ | || '_ \ / __|
*  | |     | || | | |\__ \
*  |_|     |_||_| |_||___/

* 02 - SOLENOID 6
* 03 - SOLENOID 5
* 04 - SOLENOID 4
* 05 - SOLENOID 3
* 06 - SOLENOID 2
* 07 - SOLENOID 1
* 08 - SPARE TRIGGER
* 09 - FLASH TRIGGER 2
* 10 - FLASH TRIGGER 1
* 11 - FOCUS
* 12 - SHUTTER
* 13 - 
* 14 A0 - LED - waiting. Normally yellow
* 15 A1 - LED - Active.  Normally green
* 16 A2 
* 17 A3 
* 18 A4 - SOFTWARE SERIAL TX (Bluetooth module)
* 19 A5 - SOFTWARE SERIAL RX (Bluetooth module)
* 20 A6
* 21 A7  
*
*
* 
*    _____                                                _      
*   / ____|                                              | |     
*  | |      ___   _ __ ___   _ __ ___    __ _  _ __    __| | ___ 
*  | |     / _ \ | '_ ` _ \ | '_ ` _ \  / _` || '_ \  / _` |/ __|
*  | |____| (_) || | | | | || | | | | || (_| || | | || (_| |\__ \
*   \_____|\___/ |_| |_| |_||_| |_| |_| \__,_||_| |_| \__,_||___/
* 
* 
* N12345 - number of drops
* S12345 - drop size
* D12345 - delay between drops
* E      - end of data - start the drops
* A      - abort
* Vxy - open and close valves.
* 
* [N00010][S00100][D00300][E][A]
* 
* [V11]                - Valve 1 open
* [V10]                - Valve 1 close
* [V21]                - Valve 2 open
* [V20]                - Valve 2 close
* [V31]                - Valve 3 open
* [V30]                - Valve 3 close
* 
*/


// When true, debug information is sent through the attached BT classic module.
const boolean DEBUG = false;


#include <SoftwareSerial.h> 
SoftwareSerial BTserial(18, 19); // RX, TX


const char fileName[] = __FILE__;
const char compileDate[] = __DATE__;



// this is the valve to use for the drops



// Pins

const byte LED_GREEN        = 15;
const byte LED_YELLOW       = 14;

const byte CT_SHUTTER_PIN   = 12;
const byte CT_FOCUS_PIN     = 11;

const byte FT1_PIN          = 10;
const byte FT2_PIN          = 9;
const byte FT3_PIN          = 8;

const byte ST1_PIN          = 7; 
const byte ST2_PIN          = 6; 
const byte ST3_PIN          = 5; 
const byte ST4_PIN          = 4; 
const byte ST5_PIN          = 3; 
const byte ST6_PIN          = 2; 


unsigned long   numDrops          = 0;
 byte           valvePort         = 1; 
unsigned long   delayBetweenDrops = 100;
unsigned long   dropSize          = 35;

unsigned long   tempVal           = 0;

boolean ABORT = false;


const byte numChars = 30;
char receivedChars[numChars];

boolean haveNewData = false;
boolean haveNewDrop = false;

boolean done = false;
unsigned long dropsDone = 0;

int status = 0;

int WAITING = 0;
int START = 1;
int MAKING_DROP = 2;
int DELAYING = 3;
int DROP_FINISHED = 4;
int SEQUENCE_COMPLETED = 5;


unsigned long dropStartTime = 0;
unsigned long delayStartTime = 0;

boolean SEQ_ACTIVE = false;

void setup() 
{
    Serial.begin(9600);
    while (!Serial) {;} 

    BTserial.begin(9600);  


    if(DEBUG) 
   {
     BTserial.println("");
     BTserial.print("File  = "); BTserial.println(fileName);
     BTserial.print("Date  = "); BTserial.println(compileDate);     
     BTserial.println("");
   }




     
    // status LEDs
    pinMode(LED_YELLOW,OUTPUT); digitalWrite(LED_YELLOW, LOW); 
    pinMode(LED_GREEN,OUTPUT);  digitalWrite(LED_GREEN, LOW); 
  
    // define the trigger pins and set the pins to low
    pinMode(ST1_PIN, OUTPUT);    digitalWrite(ST1_PIN, LOW); 
    pinMode(ST2_PIN, OUTPUT);    digitalWrite(ST2_PIN, LOW); 
    pinMode(ST3_PIN, OUTPUT);    digitalWrite(ST3_PIN, LOW);
    pinMode(ST4_PIN, OUTPUT);    digitalWrite(ST4_PIN, LOW); 
    pinMode(ST5_PIN, OUTPUT);    digitalWrite(ST5_PIN, LOW); 
    pinMode(ST6_PIN, OUTPUT);    digitalWrite(ST6_PIN, LOW);    
    
    pinMode(CT_FOCUS_PIN,   OUTPUT);     digitalWrite(CT_FOCUS_PIN,   LOW); 
    pinMode(CT_SHUTTER_PIN, OUTPUT);     digitalWrite(CT_SHUTTER_PIN, LOW); 
    pinMode(FT1_PIN, OUTPUT);            digitalWrite(FT1_PIN, LOW);      
    pinMode(FT2_PIN, OUTPUT);            digitalWrite(FT2_PIN, LOW);    
    pinMode(FT3_PIN, OUTPUT);            digitalWrite(FT3_PIN, LOW);   

    initialise();
}




void loop() 
{
     recvWithStartEndMarkersUSB();
     recvWithStartEndMarkersBT(); 
     if (haveNewData) { processNewData(); }  
     if (ABORT) { initialise(); }

      if (haveNewDrop)                       
      {  
           if (DEBUG) 
           { 
               BTserial.println("\n\nstatus = START"); 
               BTserial.print("\No drops = "); BTserial.println(numDrops); 
           }
           haveNewDrop = false;
           if (numDrops >0)   { status = START;  }   
           else               { 
                                status = WAITING;   
                                digitalWrite(LED_YELLOW, LOW);  delay(250);  digitalWrite(LED_YELLOW, HIGH); delay(250);
                                digitalWrite(LED_YELLOW, LOW);  delay(250);  digitalWrite(LED_YELLOW, HIGH);
                              }
           
      } //  if(haveNewDrop)

       if(status != WAITING) { makeDrops(); }
}


void makeDrops()
{
// numDrops, dropSize, delayBetweenDrops
           
           if (status == START)
           {
              digitalWrite(LED_YELLOW, LOW);    digitalWrite(LED_GREEN, HIGH);
              openSolenoid(valvePort);          dropStartTime = millis();  
              status = MAKING_DROP;          
              
              if (DEBUG) {BTserial.println("status = MAKING_DROP");}
           }

           if (status == MAKING_DROP)
           {
              if ( millis() - dropStartTime >= dropSize  )
              {
                  closeSolenoid(valvePort);
                  dropsDone++;
                  Serial.print("[C,");
                  Serial.print(dropsDone);
                  Serial.print("]");
                  
                  status = DELAYING; delayStartTime = millis();
                  if (DEBUG) {BTserial.println("status = DELAYING");}

                  if(DEBUG) {BTserial.print("dropsDone = "); BTserial.println(dropsDone);  }
              }
           }

           if (status == DELAYING)
           {
              if (millis() - delayStartTime >= delayBetweenDrops  )   
              {   
                status = START; 
                if(DEBUG) { BTserial.println("Delay finished    -  staus =  START");  }
              
              }
           }



               
          if (dropsDone >= numDrops)
          {
             status = SEQUENCE_COMPLETED;
             if (DEBUG) { BTserial.print("Sequence finished.  dropsDone = ");  BTserial.println( dropsDone);  }
             BTserial.print ("#");
             Serial.print("[#]");
             initialise();
          }

}  // makeDrops()
