
/*
****************************************
* Function initialise
* 
* reset variables
*/
void initialise()
{  
      if (DEBUG) { BTserial.println("INIT"); }
      closeSolenoid(valvePort);
      digitalWrite(LED_YELLOW, HIGH);    
      digitalWrite(LED_GREEN, LOW);
      
      numDrops = 0;
      dropsDone = 0;
      
      haveNewDrop  = false; 
      ABORT = false;
      haveNewDrop = false;
      done = false;

      status = WAITING;

}




// function recvWithStartEndMarkers by Robin2 of the Arduino forums
// See  http://forum.arduino.cc/index.php?topic=288234.0
/*
****************************************
* Function recvWithStartEndMarkers
* Reads serial data and returns the content between a start marker and an end marker.
* Lazy solution. I created 2 different versions. One to handle hardware serial and one to handle software serial
*/
void recvWithStartEndMarkersBT() 
{
     static boolean recvInProgressBT = false;
     static byte ndx = 0;
     char startMarker = '[';
     char endMarker = ']';
     char rc;
     if (BTserial.available() > 0) 
     { 
          rc = BTserial.read();
          if (recvInProgressBT == true) 
          {
               if (rc != endMarker) 
               {
                    receivedChars[ndx] = rc;
                    ndx++;
                    if (ndx >= numChars) { ndx = numChars - 1; }
               }
               else 
               {
                     receivedChars[ndx] = '\0'; // terminate the string
                     recvInProgressBT = false;
                     ndx = 0;
                     haveNewData = true;
               }
          }
          else if (rc == startMarker) { recvInProgressBT = true; }
     }
}

void recvWithStartEndMarkersUSB() 
{
     static boolean recvInProgress = false;
     static byte ndx = 0;
     char startMarker = '[';
     char endMarker = ']';
     char rc;
     if (Serial.available() > 0) 
     {
          rc = Serial.read();
          if (DEBUG) { BTserial.print(rc); }
          if (recvInProgress == true) 
          {
               if (rc != endMarker) 
               {
                    receivedChars[ndx] = rc;
                    ndx++;
                    if (ndx >= numChars) { ndx = numChars - 1; }
               }
               else 
               {
                     receivedChars[ndx] = '\0'; // terminate the string
                     recvInProgress = false;
                     ndx = 0;
                     haveNewData = true;
                     if (DEBUG) { BTserial.println(""); }
               }
          }
          else if (rc == startMarker) { recvInProgress = true; }
     }
}



/*
****************************************
* Function processNewData() 
* 
* Parse / handle recieved commands / drop data 
*/
void processNewData()  
{
    // if (DEBUG) { BTserial.println("ParseNewData");}
    
    tempVal = 0;
  
    // N  Number of drops -  [N12345]
    if (  receivedChars[0]=='N'  )
    {  
         tempVal =           (receivedChars[1] - 48) * 10000;
         tempVal = tempVal + (receivedChars[2] - 48) * 1000;
         tempVal = tempVal + (receivedChars[3] - 48) * 100;
         tempVal = tempVal + (receivedChars[4] - 48) * 10;
         tempVal = tempVal +  receivedChars[5] - 48;
         numDrops = tempVal;
         receivedChars[0] = '\0';
         if (DEBUG) { BTserial.print("num drops = ");  BTserial.println(numDrops); }
    }

    // P valvePort [Pn]  n = 1 to 6
    else if (  receivedChars[0]=='P'  )
    {  
         valvePort = receivedChars[1] - 48;
         receivedChars[0] = '\0';
         if (DEBUG) { BTserial.print("valve port = ");  BTserial.println(valvePort); }
    }



    // S  dropSize -  [S12345]
    else if (  receivedChars[0]=='S'  )
    {  
         tempVal =           (receivedChars[1] - 48) * 10000;
         tempVal = tempVal + (receivedChars[2] - 48) * 1000;
         tempVal = tempVal + (receivedChars[3] - 48) * 100;
         tempVal = tempVal + (receivedChars[4] - 48) * 10;
         tempVal = tempVal +  receivedChars[5] - 48;
         dropSize = tempVal;
         receivedChars[0] = '\0';
         if (DEBUG) { BTserial.print("drop size = ");  BTserial.println(dropSize); }
    }


    // D  ddelayBetweenDrops -  [D12345]
    else if (  receivedChars[0]=='D'  )
    {  
         tempVal =           (receivedChars[1] - 48) * 10000;
         tempVal = tempVal + (receivedChars[2] - 48) * 1000;
         tempVal = tempVal + (receivedChars[3] - 48) * 100;
         tempVal = tempVal + (receivedChars[4] - 48) * 10;
         tempVal = tempVal +  receivedChars[5] - 48;
         delayBetweenDrops = tempVal;
         receivedChars[0] = '\0';
         if (DEBUG) { BTserial.print("delay = ");  BTserial.println(delayBetweenDrops); }
    }


    // E end of drop data  [E]
    else if (  receivedChars[0]=='E'  )
    {  
         haveNewDrop = true;
         receivedChars[0] = '\0';
    }


    // A Abort  [A]
    else 
    if (  receivedChars[0]=='A'  )
    {  
         ABORT = true;
         receivedChars[0] = '\0';
    }



     // HELLO  [HELLO]
    else if (  receivedChars[0]=='H'  )
    {  
        Serial.print("[HELLO]");
      
         if (DEBUG) { BTserial.print("[HELLO]"); }
         receivedChars[0] = '\0';
    }
    
           
      // Open/close valve
      // O11 - Valve 1 open    O10 - Valve 1 close
      else if ( receivedChars[0] == 'V'   )
      {  
           byte valve = receivedChars[1] - 48;  //  -48 converts an ascii "1" in to the value 1
           byte flag =  receivedChars[2] - 48;
           
           if (valve >0 && valve < 7)
           {
                // open valve
                if (flag == 1) {  openSolenoid(valve);    }
          
                // close valve
                if (flag == 0) {  closeSolenoid(valve);   }
           }
           receivedChars[0] = '\0';
      }   

          haveNewData = false;  

} 






/*
****************************************
* Function openSolenoid
* opens a solenoid valve
* done this way so that a constant value is used.
*
*/
void openSolenoid(byte s)
{
    // done this way so I can use a constant for the sol pin
    if      (s==1) { digitalWrite(ST1_PIN, HIGH);    }
    else if (s==2) { digitalWrite(ST2_PIN, HIGH);    }
    else if (s==3) { digitalWrite(ST3_PIN, HIGH);    }
    else if (s==4) { digitalWrite(ST4_PIN, HIGH);    }    
    else if (s==5) { digitalWrite(ST5_PIN, HIGH);    }   
    else if (s==6) { digitalWrite(ST6_PIN, HIGH);    }    
    
}


/*
****************************************
* Function closeSolenoid
* closes a solenoid valve
* 
*/
void closeSolenoid(byte s)
{
     // done this way so I can use a constant for the sol pin
    if      (s==1) { digitalWrite(ST1_PIN, LOW);    }
    else if (s==2) { digitalWrite(ST2_PIN, LOW);    }
    else if (s==3) { digitalWrite(ST3_PIN, LOW);    }
    else if (s==4) { digitalWrite(ST4_PIN, LOW);    }    
    else if (s==5) { digitalWrite(ST5_PIN, LOW);    }   
    else if (s==6) { digitalWrite(ST6_PIN, LOW);    }   
}
